import pyfie
import matplotlib.pyplot as plt
import math

# 利便のため PyFIE 関数が返すエラーコードに応じて例外を発生させる機能を有効化
pyfie.ctrl.enable_f_err_exception(True)

# 処理対象画像の読み込み
image = pyfie.imread("edge1d_radial_input.png")

# 放射状エッジ検出の範囲
center = (256, 237)
start_radius = 105
range_radius = 20

# 画像オブジェクトのプロット
ax = image.imshow()
# 検出範囲をプロット
ax.add_artist(plt.Circle(center, start_radius, color="red", fill=False))
ax.add_artist(plt.Circle(center, start_radius +
                         range_radius, color="red", fill=False))
# プロットした画像の表示
pyfie.plot_show()

# データ受け取り用変数を用意
num_edges = pyfie.INT(8)
edges = pyfie.F_DEDGE.ARRAY(num_edges)

# 放射状エッジ検出
pyfie.edge1d_radial(
    hsrc=image,  # 入力画像
    center=center,  # 計測円の中心座標
    start_radius=start_radius,  # 検出ラインの開始半径
    range_radius=range_radius,  # 検出ラインの処理範囲半径
    start_angle=math.pi / num_edges,  # 開始角度
    range_angle=2 * math.pi * (1 - 1 / num_edges),  # 処理範囲角度
    line_num=num_edges,  # 検出ライン本数
    prj_mode=pyfie.F_PROJECTION_FAN,  # 濃度投影方法
    prj_width=5,  # 濃度投影片幅
    filter_mode=pyfie.F_EDGE1D_DOG,  # エッジ抽出フィルタ
    direct_mode=pyfie.F_DRK_TO_BRI,  # 検出するエッジ方向
    diff_type=pyfie.F_ABSOLUTE_THR,  # 濃度差しきい値の種別
    diff_thr=50,  # エッジの最小濃度差しきい値
    detect_mode=pyfie.F_POS_SORT,  # エッジ点の出力方法
    edges=edges.ref,  # エッジ点保存配列
    edge_num=num_edges  # 検出エッジ点個数
)

# 取得したエッジを表示
print("num edges:", num_edges)
for i, edge in enumerate(edges[:num_edges]):
    print("#{}: x = {:.2f}, y = {:.2f}, mag = {:.2f}".format(
        i, edge.x, edge.y, edge.mag
    ))

# 画像オブジェクトのプロット
image.imshow()
# 取得したエッジをプロット
edges.plot(c="red", num=num_edges, s=100)
# 結果をプロットした画像の保存
plt.savefig("edge1d_radial_result.png")
# 結果をプロットした画像の表示
pyfie.plot_show()
